/*********************************************************************** 
 * $Id: main.c 8651 2011-11-18 10:07:27Z nxp28536 $
 * 
 * Project: Mailbox M4<->M0 Application Example for LPC43xx 
 * 
 * Description: Implements a Mailbox application
 * 
 * Copyright(C) 2011, NXP Semiconductor
 * All rights reserved.
 * 
 ***********************************************************************
 * Software that is described herein is for illustrative purposes only  
 * which provides customers with programming information regarding the  
 * products. This software is supplied "AS IS" without any warranties.  
 * NXP Semiconductors assumes no responsibility or liability for the 
 * use of the software, conveys no license or title under any patent, 
 * copyright, or mask work right to the product. NXP Semiconductors 
 * reserves the right to make changes in the software without 
 * notification. NXP Semiconductors also make no representation or 
 * warranty that such application will be suitable for the specified 
 * use without further testing or modification. 
 **********************************************************************/
#ifndef CORE_M0
	  #error "Build Error: please define CORE_M0 in the project settings" 
#endif

#include "LPC43xx.h"
#include "lpc43xx_scu.h"
#include "lpc43xx_cgu.h"
#include "debug_frmwrk.h"

#include "ipc_mbx.h"
#include "ipc_memory.h"
#include "platform_check.h"
#include "tcpip.h"
#include "stdlib.h"
#include "string.h"
#include "stdio.h"

void vIOInit(void);
void initRiTimer(uint32_t timerInterval);
void initTimer0(uint32_t TimerInterval);

mbxParam_t returnString;
mbxParam_t returnMessages;
mbxParam_t returnPower;
uint32_t localBase = 0;

volatile uint8_t sendA = 0;
volatile uint8_t sendB = 0;
volatile uint8_t sendC = 0;

void web_main(void);
void web_tick(void);

//==============================================================================
//   Main Function (M0)
//==============================================================================
#define CLOCK_SPEED (204000000)

int main(void)
{					
	vIOInit();

	debug_frmwrk_init_clk(CLOCK_SPEED);

	_DBG("\n\r--- M0 Started ---\n\r");

	/* SysTick is not implemented on M0, use RITIMER */
	initRiTimer(CLOCK_SPEED/1000);

	ipc_PutValue(IPC_IP_ADDRESS, MY_IP);
	ipc_PutValue(IPC_M0_COUNTER, 1);

	web_main();
}

//==============================================================================
//  Initialize board specific I/O
//==============================================================================

void vIOInit(void)
{
	#if (PLATFORM == NXP_VALIDATION_BOARD)
		// PA.4 : GPIO5_19: LD10 (LED)
		scu_pinmux(0xA ,4 , MD_PDN, FUNC4); 	
		LPC_GPIO_PORT->DIR[5] |= (1UL << 19);
	#endif
	#if (PLATFORM == HITEX_A2_BOARD)
		// P8.1 : GPIO4_1
		scu_pinmux(0x8 ,1 , MD_PDN, FUNC0); 	
		LPC_GPIO_PORT->DIR[4] |= (1UL << 1);
	#endif
}

//==============================================================================
// System tick interrupt handler
//==============================================================================

static volatile uint32_t u32Milliseconds = 10;

void M0_RIT_OR_WWDT_IRQHandler (void) 					
{           
	LPC_RITIMER->CTRL |= (1<<0);			// clear interrupt flag

	if (u32Milliseconds > 0)
		u32Milliseconds--;

	else
	{
		u32Milliseconds = 10;
		web_tick();							// handle web activities
	}
}

//==============================================================================
// Initialize RIT timer
//==============================================================================

void initRiTimer(uint32_t TimerInterval)
{
	LPC_RITIMER->COMPVAL = TimerInterval; 			// Set match value
	LPC_RITIMER->COUNTER=0;							// Set count value to 0
	LPC_RITIMER->CTRL = (1<<3)|(1<<2)|(1<<1);		// Enable, enable break, reset on mask compare, clear interrupt flag

	NVIC_EnableIRQ(M0_RITIMER_OR_WWDT_IRQn);
}


//==============================================================================
// Initialize M0 timer	0
//==============================================================================

#define HZ1 	1000
#define PERIOD	5

void initTimer0(uint32_t TimerInterval)
{
	
	LPC_TIMER0->PR = HZ1; // scale down to 1HZ = 120000
	
	// period 3Hz, reset
	LPC_TIMER0->MR[3] = (TimerInterval) * PERIOD; 

	// these give interrupts
	LPC_TIMER0->MR[2] = (90*PERIOD*HZ1);
	LPC_TIMER0->MR[1] = (60*PERIOD*HZ1);
	LPC_TIMER0->MR[0] = (30*PERIOD*HZ1);

	LPC_TIMER0->MCR = (((0x2) << 9)) | (((0x1) << 6)) | (((0x1) << 3)) | ((0x1) << 0);

	LPC_TIMER0->TC = 0;	

	LPC_TIMER0->IR = 0xFF;	

	NVIC_EnableIRQ(TIMER0_IRQn);
	
	LPC_TIMER0->TCR = 0x1;	 // start the timer
}

//==============================================================================
// M0 timer 0 interrupt handler
//==============================================================================

void M0_TIMER0_IRQHandler(void)
{
	// check which match sent the interrupt
	if(LPC_TIMER0->IR & 0x1)
	{
		LPC_TIMER0->IR = 0x1;
		sendA = 1;
	}

	if(LPC_TIMER0->IR & 0x2)
	{
		LPC_TIMER0->IR = 0x2;
		sendB = 1;
	}

	if(LPC_TIMER0->IR & 0x4)
	{
		LPC_TIMER0->IR = 0x4;
		sendC = 1;
	}
}

/*******************************************************************************
* @brief		Reports the name of the source file and the source line number
* 				where the CHECK_PARAM error has occurred.
* @param[in]	file Pointer to the source file name
* @param[in]    line assert_param error line source number
* @return		None
*******************************************************************************/
void check_failed(uint8_t *file, uint32_t line)
{
	/* User can add his own implementation to report the file name and line number,
	 ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

	/* Infinite loop */
	while(1);
}

/*****************************************************************************
**                            End Of File
*****************************************************************************/
