/**********************************************************************
*
* @file		demo_utilities.c
* @brief	Demo utilities.
* @version	1.0
* @date		11. January. 2012
* @author	Cam Thompson, Micromega Corporation
*
**********************************************************************/

#include "lpc43xx_rtc.h"
#include "pca9502.h"
#include "COG_LCD.h"
#include "string.h"
#include "demo_Utilities.h"

extern uint32_t msec;

static void ChangeField(uint8_t field, uint8_t touch);
static void DrawField(uint8_t field, uint16_t value, Bool selected);

static uint8_t value[6] = {12, 1, 1, 12, 0, 0};
static int8_t n;

char dateTime[20];

//==============================================================================
// Get date/time from COG LCD and set RTC
//==============================================================================

void SetDateTime(void)
{
	uint8_t lastTouch, currentTouch, change, field;

	RTC_Cmd(LPC_RTC, DISABLE);

	// prompt for date/time entry
	COG_LCD_Clear();
	COG_LCD_DrawString(0, 0, "Set Date/Time:");
	COG_LCD_DrawString(8, 10, "2012-01-01 12:00:00");

	// display buttons
	DrawButton(0, "+");
	DrawButton(1, "-");
	DrawButton(2, "Next");
	DrawButton(3, "Set");
	COG_LCD_Update();

	field = 0;
	DrawField(field, value[field], TRUE);
   	COG_LCD_Update();

	lastTouch = ReadTouch();
	pca9502_WriteLeds(~lastTouch);
	change = lastTouch;
		
	while (1)
	{
		currentTouch = ReadTouch();
		pca9502_WriteLeds(~currentTouch);
		change = lastTouch ^ currentTouch;
		lastTouch = currentTouch;

		if (change)
		{
		   	COG_LCD_PenType(PEN_TOGGLE);
			if (change & TOUCH_T4) COG_LCD_FillRect(1, 21, 28, 9);
			if (change & TOUCH_T3) COG_LCD_FillRect(32+1, 21, 28, 9);
			if (change & TOUCH_T2) COG_LCD_FillRect(64+1, 21, 28, 9);
			if (change & TOUCH_T1) COG_LCD_FillRect(96+1, 21, 28, 9);

			if (currentTouch & (TOUCH_T3 | TOUCH_T4))	// +/-
				ChangeField(field, currentTouch);
				COG_LCD_Update();

			if (currentTouch & TOUCH_T2)			// Next
			{
				DrawField(field, value[field], FALSE);
			 	if (++field > 5) field = 0;
				DrawField(field, value[field], TRUE);
				COG_LCD_Update();
			}
			else if (currentTouch & TOUCH_T1)			// Set
			{
				RTC_SetTime(LPC_RTC, RTC_TIMETYPE_YEAR, 2000 + value[0]);
				RTC_SetTime(LPC_RTC, RTC_TIMETYPE_MONTH, value[1]);
				RTC_SetTime(LPC_RTC, RTC_TIMETYPE_DAYOFMONTH, value[2]);
				RTC_SetTime(LPC_RTC, RTC_TIMETYPE_HOUR, value[3]);
				RTC_SetTime(LPC_RTC, RTC_TIMETYPE_MINUTE, value[4]);
				RTC_SetTime(LPC_RTC, RTC_TIMETYPE_SECOND, value[5]);

				COG_LCD_FillRect(96+1, 21, 28, 9);
				DrawField(field, value[field], FALSE);
				COG_LCD_Update();

				RTC_Cmd(LPC_RTC, ENABLE);
				return;
			}
		}
	}
}

//==============================================================================
// get date value from RTC
//==============================================================================

int DateValue(void)
{
	return (RTC_GetTime(LPC_RTC, RTC_TIMETYPE_YEAR) << 16)
		+ (RTC_GetTime(LPC_RTC, RTC_TIMETYPE_MONTH) << 8)
		+ RTC_GetTime(LPC_RTC, RTC_TIMETYPE_DAYOFMONTH);
}

//==============================================================================
// get time value from RTC
//==============================================================================

int TimeValue(void)
{
	return (RTC_GetTime(LPC_RTC, RTC_TIMETYPE_HOUR) << 16)
		+ (RTC_GetTime(LPC_RTC, RTC_TIMETYPE_MINUTE) << 8)
		+ RTC_GetTime(LPC_RTC, RTC_TIMETYPE_SECOND);
}

//==============================================================================
// get DateTime stamp
//==============================================================================

char *DateTimeString(int date, int time)
{
	char s2[5];

	itoa((date>>16)&0xFFFF, dateTime, 4);
	strcat(dateTime, "-");
	strcat(dateTime, itoa((date>>8)&0xFF, s2, 2));
	strcat(dateTime, "-");
	strcat(dateTime, itoa(date&0xFF, s2, 2));
	strcat(dateTime, " ");
	strcat(dateTime, itoa((time>>16)&0xFF, s2, 2));
	strcat(dateTime, ":");
	strcat(dateTime, itoa((time>>8)&0xFF, s2, 2));
	strcat(dateTime, ":");
	strcat(dateTime, itoa(time&0xFF, s2, 2));
	return dateTime;
}

//==============================================================================
// draw field on COG LCD display
//==============================================================================

static void DrawField(uint8_t field, uint16_t value, Bool selected)
{
	char s[3];

	field = 19 + (field * 18);
	COG_LCD_PenType(selected ? PEN_SET : PEN_CLEAR);
	COG_LCD_FillRect(field, 9, 13, 9);
	COG_LCD_PenType(selected ? PEN_CLEAR : PEN_SET);
	COG_LCD_DrawString(field+1, 10, itoa(value, s, 2));
}

//==============================================================================
// change field on COG LCD display
//==============================================================================

static void ChangeField(uint8_t field, uint8_t touch)
{
	msec = 500;		// delay auto-repeat for 1/2 second
	while (1)
	{
		if (touch & TOUCH_T3)
			n = value[field] - 1;
		else
			n = value[field] + 1;

		switch (field)
		{
		case 0: //year
			if (n < 0)
				n = 99;
			else if (n > 99)
				n = 0;
			break;

		case 1: //month
			if (n < 1)
				n = 12;
			else if (n > 12)
				n = 1;
			break;

		case 2:	//day
			if (n < 1)
				n = 31;
			else if (n > 31)
				n = 1;
			break;

		case 3:	// hour
			if (n < 0)
				n = 23;
			else if (n > 23)
				n = 0;
			break;

		default:
			if (n < 0)
				n = 59;
			else if (n > 59)
				n = 0;
		}
		value[field] = n;
		DrawField(field, n, TRUE);

		COG_LCD_Update();

		while (msec)
		{
			touch = ReadTouch();
			if ((touch & (TOUCH_T3 | TOUCH_T4)) == 0) return;
		}
		msec = 200; 	// auto-repeat every 1/10 second
	}
}

//==============================================================================
// draw a button on the COG LCD screen
//==============================================================================

void DrawButton(uint8_t button, char *title)
{
	uint8_t x;

	if (button > 3) return;
	button *= 32;
	COG_LCD_DrawRect(button, 20, 30, 11);
	COG_LCD_DrawLine(button+1, 31, button+30, 31);
	COG_LCD_DrawLine(button+30, 21, button+30, 31);
	x = (30 - (strlen(title) * 6)) / 2;
	COG_LCD_DrawString(button+x, 22, title);
}

//==============================================================================
// integer to string
//==============================================================================

char *itoa(int value, char *s, uint8_t len)
{
	char *sptr;

	sptr = &s[len];
	*sptr-- = '\0';
	while (len-- > 0)
	{
		*sptr-- = '0' + value % 10;
		value /= 10;
	}
	return s;	
}
