/**********************************************************************
*
* @file		pca9502.h
* @brief	Header file for PCA9502 Touch keys and LEDs
* @version	1.0
* @date		09. January. 2012
* @author	Cam Thompson, Micromega Corporation
*
**********************************************************************/

#ifndef _PCA9502_H_
#define _PCA9502_H_

/* Includes ------------------------------------------------------------------- */
#include "lpc_types.h"

/* Public Declarations -------------------------------------------------------- */

// PCA9502 declarations
#define PCA9502_ADDR	(0x9A>>1)

#define PCA9502_IODIR	(0x0A<<3)
#define PCA9502_IOSTATE	(0x0B<<3)
#define PCA9502_IOINTEN	(0x0C<<3)
#define PCA9502_IOCTRL	(0x0E<<3)

#define PCA9502_RESET	(1<<3)

#define PCA9502_LED0	8
#define PCA9502_LED1	4
#define PCA9502_LED2	2
#define PCA9502_LED3	1

#define TOUCH_T1		8
#define TOUCH_T2		4
#define TOUCH_T3		2
#define TOUCH_T4		1
#define TOUCH_ANY		0x0F

/* Public Functions ----------------------------------------------------------- */

void pca9502_Init(void);
void pca9502_WriteLeds(uint8_t led);
void pca9502_SetLed(uint8_t led);
void pca9502_ClearLed(uint8_t led);
void pca9502_ToggleLed(uint8_t led);
uint8_t ReadTouch(void);
uint8_t WaitTouch(uint8_t touch);

#endif // _PCA9502_H_
