/**********************************************************************
*
* @file		ipc_memory.c
* @brief	Simple shared memory interprocessor communication.
* @version	1.0
* @date		January 20, 2012
* @author	Cam Thompson, Micromega Corporation
*
**********************************************************************/

#include "lpc_types.h"
#include "ipc_memory.h"
#include "string.h"

//==============================================================================
// Clear all inter-process communication values
//==============================================================================

void ipc_Init(void)
{
	int i;

	for (i = 0; i < IPC_MAX_WORDS; i++)
	{
		ipc_PutValue(i, 0);
	}
}

//==============================================================================
// Start IPC write sequence
//==============================================================================

void ipc_StartWrite(void)
{
	while (IPC_READ) ;	// wait for IPC read flag = 0
	IPC_WRITE = 1;		// set write flag
}

//==============================================================================
// End IPC write sequence
//==============================================================================

void ipc_EndWrite(void)
{
	IPC_WRITE = 0;		// clear write flag
}

//==============================================================================
// Start IPC read sequence
//==============================================================================

void ipc_StartRead(void)
{
	while (IPC_WRITE) ;	// wait for IPC write flag = 0
	IPC_READ = 1;		// set read flag
}

//==============================================================================
// End IPC write sequence
//==============================================================================

void ipc_EndRead(void)
{
	IPC_READ = 0;		// clear read flag
}

//==============================================================================
// Get inter-process communication value
//==============================================================================

void ipc_PutValue(int idx, int value)
{
	int	*ptr;

	if (idx < IPC_MAX_WORDS)
	{
		ptr = (int *)IPC_WORD_START;
		ptr[idx] = value;
	}
}

//==============================================================================
// Put inter-process communication value
//==============================================================================

int ipc_GetValue(int idx)
{
	int	*ptr;

	if (idx < IPC_MAX_WORDS)
	{
		ptr = (int *)IPC_WORD_START;
		return ptr[idx];
	}
	else
		return 0;
}

//==============================================================================
// Increment inter-process communication value
//==============================================================================

void ipc_IncrementValue(int idx)
{
	int	*ptr;

	if (idx < IPC_MAX_WORDS)
	{
		ptr = (int *)IPC_WORD_START;
		ptr[idx]++;
	}
}


