/**********************************************************************
*
* @file		ipc_memory.h
* @brief	Simple shared memory interprocessor communication.
* @version	1.0
* @date		January 20, 2012
* @author	Cam Thompson, Micromega Corporation
*
**********************************************************************/

// shared memory starting at 0x20000000
#define IPC_MAX_WORDS		32
#define IPC_WORD_START		0x20000000

// index number for IPC variables
#define IPC_M4_COUNTER		0
#define IPC_M0_COUNTER		1
#define IPC_IP_ADDRESS		2
#define IPC_WRITE_FLAG		3
#define IPC_READ_FLAG		4
#define IPC_SAMPLE_COUNT	5
#define IPC_SAMPLE_DATE		6
#define IPC_SAMPLE_TIME		7
#define IPC_TEMP			8
#define IPC_ADC				9
#define IPC_AVE_TEMP		10
#define IPC_MIN_TEMP		11
#define IPC_MAX_TEMP		12
#define IPC_AVE_ADC			13
#define IPC_MIN_ADC			14
#define IPC_MAX_ADC			15
#define IPC_ALARM_COUNT		16
#define IPC_ALARM_DATE		17
#define IPC_ALARM_TIME		18

// direct access to M4/M0 counters
#define	M4_COUNTER		*(volatile int *) 0x20000000
#define	M0_COUNTER		*(volatile int *) 0x20000004
#define	IPC_WRITE		*(volatile int *) 0x2000000C
#define	IPC_READ		*(volatile int *) 0x20000010

void ipc_Init(void);
void ipc_StartWrite(void);
void ipc_EndWrite(void);
void ipc_StartRead(void);
void ipc_EndRead(void);
int ipc_GetValue(int idx);
void ipc_PutValue(int idx, int value);
void ipc_IncrementValue(int idx);

